    #!/bin/sh
    #================================================================================
    # (C)2013 dMajo
    # Title       : ad-blocker.sh
    # Version     : V2.00.0002
    # Orig Author : dMajo (http://forum.synology.com/enu/memberlist.php?mode=viewprofile&u=69661)
    # Description : Script to block add-banner servers, dns based
    # Dependencies: Syno DNSServer package, sed, wget
    # Usage       : sh ad-blocker.sh
    #================================================================================
    # Version history:
    # 2013.09.01 - 1.00.0001: Initial release
    # 2013.09.08 - 1.00.0004: Fix: changed include target to support views
    # 2013.09.12 - 1.00.0005: Added automatic zone file generation and some basic error handling
    # 2014.03.29 - 1.01.0013: Added dependencies check
    # 2014.03.30 - 1.02.0017: Script reorganized
    # 2014.04.06 - 1.02.0018: Fix: fixed serial number in zone file generation
	#
	# Edits by	  : SmiGueL
	# 2016.09.08 - 			  Adding blacklist + whitelist feature
	# 2016.09.15 - 			  Adding multiple websites to download hostslists from
	# 2016.09.16 - 2.00.0001  Fixed serial number error to prevent log errors when updating more than once a day
	# 2016.09.29 - 2.00.0002  Added Quidups's tracker blocklist
	#
	# Edit by     : Briolet
	# 2017.02.29 - 2.00.0003  Also export the final list to the temp folder
	# 2017.09.17 - 2.00.0004  Delete temp files + add white & black list if not pressent
	# 2017.11.20 - 2.00.0005  Allow comment and empty lines in the White and Black list	
	# 2018.01.15 - 2.00.0006  Remove duplicate malwaredomains. Try an other mirror of malwaredomains on error.
	# 						  
    #================================================================================
	
    # Define variables
	CustomLists="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/" # path to this script for storing the white & blacklists
	
	YoyoList=1									#3k			#1 to enable or 0 to disable | Automatically import updated version of the yoyo.org blocklist							http://pgl.yoyo.org/as/serverlist.php?hostformat=bindconfig&showintro=0&mimetype=plaintext
	Malwaredomains=1							#17k		#1 to enable or 0 to disable | Automatically import updated version of the malwaredomains.com blocklist					http://mirror2.malwaredomains.com/files/spywaredomains.zones
	RandomLargeList=0							#34k		#1 to enable or 0 to disable | Automatically import a large Github blocklist (last updated  )							https://raw.githubusercontent.com/mat1th/Dns-add-block/master/adresses
	PiHoleList=0								#27k		#1 to enable or 0 to disable | Automatically import updated version of the official Pi-Hole blocklist					https://raw.githubusercontent.com/StevenBlack/hosts/master/hosts
															#| This Pi-Hole list includes: a custom list + malwaredomainlist.com + WinHelp2002 + someonewhocares + yoyo.org 		https://github.com/pi-hole/pi-hole/blob/master/adlists.default
                                                            #| For other hostfiles see https://github.com/StevenBlack/hosts
	sysct1=0									#21k		#1 to enable or 0 to disable | Automatically import updated version of sysct1 blocklist									http://sysctl.org/cameleon/hosts
	amazonws1=0									#0k			#1 to enable or 0 to disable | Automatically import updated version of amazonsw blocklist								https://s3.amazonaws.com/lists.disconnect.me/simple_tracking.txt
	amazonws2=0									#3k			#1 to enable or 0 to disable | Automatically import updated version of amazonsw blocklist								https://s3.amazonaws.com/lists.disconnect.me/simple_ad.txt
	HostsFileNet=0								#48k		#1 to enable or 0 to disable | Automatically import updated version of hosts-file.net blocklist							https://hosts-file.net/ad_servers.txt
	Quidsupstracker=0							#11k		#1 to enable or 0 to disable | Automatically import updated version of Quidups's tracker blocklist						https://raw.githubusercontent.com/quidsup/notrack/master/trackers.txt
	fakenews=1							        #11k		#1 to enable or 0 to disable | Automatically import updated version of Fakenews blocklist						        https://github.com/marktron/fakenews
	
	Cleanuptemp=1        # Delete al intermediate files. Disable for debugging purposes

    # Delete temp files, except the last one
    Cleanup () {
        a=$1
        while [ "$a" -lt $2 ]; do
           rm ${CustomLists}/temp/$3${a}.txt
           a=`expr $a + 1`
        done
    }
	
	
    RootDir="/var/packages/DNSServer/target"
    ZoneDir="${RootDir}/named/etc/zone"
    ZoneDataDir="${ZoneDir}/data"
    ZoneMasterDir="${ZoneDir}/master"
    ScriptLog="${CustomLists}/ad-blocker.log"


    cd ${ZoneDataDir}
    # Check if needed dependencies exists
       Dependencies="chown chmod date grep mv rm sed wget"
       MissingDep=0
       for NeededDep in $Dependencies; do
          if ! hash "$NeededDep" >/dev/null 2>&1; then
             printf "Command not found in PATH: %s\n" "$NeededDep" >&2
             MissingDep=$((MissingDep+1))
          fi
       done
       if [ $MissingDep -gt 0 ]; then
          printf "Minimum %d commands are missing in PATH, aborting\n" "$MissingDep" >&2
          exit 1
       fi

	mkdir -p ${CustomLists}/temp
    rm -f ad-blocker.db
    > ad-blocker.db
    chown -R nobody:nobody ad-blocker.db

if [ "$YoyoList" -eq 1 ] && [ "$PiHoleList" -eq 0 ]; then
    # Download the "blacklist" from "http://pgl.yoyo.org"
    wget "http://pgl.yoyo.org/as/serverlist.php?hostformat=bindconfig&showintro=0&mimetype=plaintext"

    # Modify Zone file path from "null.zone.file" to "/etc/zone/master/null.zone.file" in order to comply with Synology bind implementation
    sed -e 's/null.zone.file/\/etc\/zone\/master\/null.zone.file/g' "serverlist.php?hostformat=bindconfig&showintro=0&mimetype=plaintext" > ${CustomLists}/temp/yoyo.txt
    rm "serverlist.php?hostformat=bindconfig&showintro=0&mimetype=plaintext"
    cat ${CustomLists}/temp/yoyo.txt >> ad-blocker.db
fi



if [ "$Malwaredomains" -eq 1 ] && [ "$PiHoleList" -eq 0 ]; then
	# Download blocklist from malwaredomains.com
	wget "http://mirror2.malwaredomains.com/files/spywaredomains.zones"
	if [ $? -ne 0 ]; then  # on error try the other mirror site
	    wget "http://mirror1.malwaredomains.com/files/spywaredomains.zones"
	fi
	sed -e 's/  {type master; file "\/etc\/namedb\/blockeddomain.hosts";}; / { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' "spywaredomains.zones" > ${CustomLists}/temp/malwaredomains10.txt
	rm "spywaredomains.zones"
	echo "" >> ${CustomLists}/temp/malwaredomains10.txt
	sed -e "s/\r/\n/g" ${CustomLists}/temp/malwaredomains10.txt > ${CustomLists}/temp/malwaredomains11.txt
	sed -e "s/\n\n/\n/g" ${CustomLists}/temp/malwaredomains11.txt > ${CustomLists}/temp/malwaredomains12.txt
	sed '/^\s*$/d' ${CustomLists}/temp/malwaredomains12.txt > ${CustomLists}/temp/malwaredomains13.txt 		# Remove all empty lines
	sed '/_/d' ${CustomLists}/temp/malwaredomains13.txt > ${CustomLists}/temp/malwaredomains14.txt	# Remove all lines with underscore chars
	
	# Add the malwaredomainlist to the blocklist
	cat ${CustomLists}/temp/malwaredomains14.txt >> ad-blocker.db

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 10 14 malwaredomains
    fi
fi


if [[ "$RandomLargeList" -eq 1 ]]; then
	# Download random blocklist from github (not updated)
	wget "https://github.com/mat1th/Dns-add-block/blob/master/adresses?raw=true"
	mv adresses?raw=true ${CustomLists}/temp/randomlargelist10.txt
	echo "" >> ${CustomLists}/temp/randomlargelist10.txt
	
	sed -e "s/\r/\n/g" ${CustomLists}/temp/randomlargelist10.txt > ${CustomLists}/temp/randomlargelist11.txt
	sed -e "s/\n\n/\n/g" ${CustomLists}/temp/randomlargelist11.txt > ${CustomLists}/temp/randomlargelist12.txt
	sed -e s/"  "/" "/g ${CustomLists}/temp/randomlargelist12.txt > ${CustomLists}/temp/randomlargelist13.txt
	sed '/_/d' ${CustomLists}/temp/randomlargelist13.txt > ${CustomLists}/temp/randomlargelist14.txt	# Remove all lines with underscore chars
	sed '/:/d' ${CustomLists}/temp/randomlargelist14.txt > ${CustomLists}/temp/randomlargelist15.txt	# Remove all lines with colon
	echo "" >> ${CustomLists}/temp/randomlargelist15.txt
	sort ${CustomLists}/temp/randomlargelist15.txt | uniq > ${CustomLists}/temp/randomlargelist16.txt
	

	cat ${CustomLists}/temp/randomlargelist16.txt >> ad-blocker.db
	
    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 10 16 randomlargelist
    fi
fi


if [[ "$PiHoleList" -eq 1 ]]; then
	# Download blocklist from Pi-Hole
	wget "https://raw.githubusercontent.com/StevenBlack/hosts/master/hosts"
	mv hosts ${CustomLists}/temp/piholelist0.txt

	sed -e '/^0.0.0.0/!d' ${CustomLists}/temp/piholelist0.txt> ${CustomLists}/temp/piholelist1.txt	# Remove all lines that does not start with 0.0.0.0
	sed 's/#.*//' ${CustomLists}/temp/piholelist1.txt > ${CustomLists}/temp/piholelist2.txt	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/piholelist2.txt > ${CustomLists}/temp/piholelist3.txt	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/_/d' ${CustomLists}/temp/piholelist3.txt > ${CustomLists}/temp/piholelist4.txt	# Remove all lines with underscore chars
	sed -e 's/0.0.0.0 /zone "/g' ${CustomLists}/temp/piholelist4.txt > ${CustomLists}/temp/piholelist5.txt	# Replace beginning of line 0.0.0.0  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/piholelist5.txt > ${CustomLists}/temp/piholelist6.txt	# Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };

	cat ${CustomLists}/temp/piholelist6.txt >> ad-blocker.db	# Add the spywaredomainlist to the blocklist

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 0 6 piholelist
	fi
fi



if [[ "$sysct1" -eq 1 ]]; then
	# Download blocklist from sysct1.com
	wget "http://sysctl.org/cameleon/hosts"
	mv hosts ${CustomLists}/temp/sysct1.txt
	echo "" >> ${CustomLists}/temp/sysct1.txt
	
	sed -e '/^127.0.0.1\t /!d' ${CustomLists}/temp/sysct1.txt> ${CustomLists}/temp/sysct2.txt 	# Remove all lines that does not start with 127.0.0.1
	sed 's/#.*//' ${CustomLists}/temp/sysct2.txt > ${CustomLists}/temp/sysct3.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/sysct3.txt > ${CustomLists}/temp/sysct4.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/_/d' ${CustomLists}/temp/sysct4.txt > ${CustomLists}/temp/sysct5.txt	# Remove all lines with underscore chars
	sed -e 's/127.0.0.1\t /zone "/g' ${CustomLists}/temp/sysct5.txt > ${CustomLists}/temp/sysct6.txt 	# Replace beginning of line 127.0.0.1  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/sysct6.txt > ${CustomLists}/temp/sysct7.txt # Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };
	
	cat ${CustomLists}/temp/sysct7.txt >> ad-blocker.db

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 1 7 sysct
	fi
fi



if [[ "$amazonws1" -eq 1 ]]; then
	# Download blocklist from amazonanews.com
	wget "https://s3.amazonaws.com/lists.disconnect.me/simple_tracking.txt"
	mv simple_tracking.txt ${CustomLists}/temp/amazonaws10.txt
	echo "" >> ${CustomLists}/temp/amazonaws10.txt
	
	sed 's/#.*//' ${CustomLists}/temp/amazonaws10.txt > ${CustomLists}/temp/amazonaws11.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/amazonaws11.txt > ${CustomLists}/temp/amazonaws12.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/^\s*$/d' ${CustomLists}/temp/amazonaws12.txt > ${CustomLists}/temp/amazonaws13.txt 		# Remove all empty lines
	sed '/_/d' ${CustomLists}/temp/amazonaws13.txt > ${CustomLists}/temp/amazonaws14.txt	# Remove all lines with underscore chars
	sed -e 's/^/zone "/g' ${CustomLists}/temp/amazonaws14.txt > ${CustomLists}/temp/amazonaws15.txt 	# Replace beginning of line 127.0.0.1  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/amazonaws15.txt > ${CustomLists}/temp/amazonaws16.txt # Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };
	
	cat ${CustomLists}/temp/amazonaws16.txt >> ad-blocker.db

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 10 16 amazonaws
	fi
fi



if [[ "$amazonws2" -eq 1 ]]; then
	# Download blocklist from amazonanews.com
	wget "https://s3.amazonaws.com/lists.disconnect.me/simple_ad.txt"
	mv simple_ad.txt ${CustomLists}/temp/amazonaws20.txt
	echo "" >> ${CustomLists}/temp/amazonaws20.txt
	
	sed 's/#.*//' ${CustomLists}/temp/amazonaws20.txt > ${CustomLists}/temp/amazonaws21.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/amazonaws21.txt > ${CustomLists}/temp/amazonaws22.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/^\s*$/d' ${CustomLists}/temp/amazonaws22.txt > ${CustomLists}/temp/amazonaws23.txt 		# Remove all empty lines
	sed '/_/d' ${CustomLists}/temp/amazonaws23.txt > ${CustomLists}/temp/amazonaws24.txt	# Remove all lines with underscore chars
	sed -e 's/^/zone "/g' ${CustomLists}/temp/amazonaws24.txt > ${CustomLists}/temp/amazonaws25.txt 	# Replace beginning of line 127.0.0.1  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/amazonaws25.txt > ${CustomLists}/temp/amazonaws26.txt # Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };
	
	cat ${CustomLists}/temp/amazonaws26.txt >> ad-blocker.db

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 20 26 amazonaws
	fi
fi



if [[ "$HostsFileNet" -eq 1 ]]; then
	# Download blocklist from hosts-file.net
	wget "https://hosts-file.net/ad_servers.txt"
	mv ad_servers.txt ${CustomLists}/temp/hostsfilenet0.txt

	sed -e "s/\r/\n/g" ${CustomLists}/temp/hostsfilenet0.txt > ${CustomLists}/temp/hostsfilenet1.txt
	sed -e "s/\n\n/\n/g" ${CustomLists}/temp/hostsfilenet1.txt > ${CustomLists}/temp/hostsfilenet2.txt
	sed '/^\s*$/d' ${CustomLists}/temp/hostsfilenet2.txt > ${CustomLists}/temp/hostsfilenet3.txt 		# Remove all empty lines
	sed -e '/^127.0.0.1\t/!d' ${CustomLists}/temp/hostsfilenet3.txt> ${CustomLists}/temp/hostsfilenet4.txt	# Remove all lines that does not start with 127.0.0.1 + tab
	sed 's/#.*//' ${CustomLists}/temp/hostsfilenet4.txt > ${CustomLists}/temp/hostsfilenet5.txt	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/hostsfilenet5.txt > ${CustomLists}/temp/hostsfilenet6.txt	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/_/d' ${CustomLists}/temp/hostsfilenet6.txt > ${CustomLists}/temp/hostsfilenet7.txt	# Remove all lines with underscore chars
	sed 's/\.$//' ${CustomLists}/temp/hostsfilenet7.txt > ${CustomLists}/temp/hostsfilenet8.txt 	# Remove all dots at the end of the line (Needed because of theoads.com. website....)
	sed -e 's/127.0.0.1\t/zone "/g' ${CustomLists}/temp/hostsfilenet8.txt > ${CustomLists}/temp/hostsfilenet9.txt	# Replace beginning of line 127.0.0.1  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/hostsfilenet9.txt > ${CustomLists}/temp/hostsfilenet10.txt	# Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };

	cat ${CustomLists}/temp/hostsfilenet10.txt >> ad-blocker.db	# Add the spywaredomainlist to the blocklist

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 0 10 hostsfilenet
	fi
fi



if [[ "$Quidsupstracker" -eq 1 ]]; then
	# Download tracker blocklist from Quidups
	wget "https://raw.githubusercontent.com/quidsup/notrack/master/trackers.txt"
	mv trackers.txt ${CustomLists}/temp/quidups1.txt
	echo "" >> ${CustomLists}/temp/quidups1.txt
	
	sed 's/#.*//' ${CustomLists}/temp/quidups1.txt > ${CustomLists}/temp/quidups2.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/quidups2.txt > ${CustomLists}/temp/quidups3.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/^\s*$/d' ${CustomLists}/temp/quidups3.txt > ${CustomLists}/temp/quidups4.txt 		# Remove all empty lines
	sed '/_/d' ${CustomLists}/temp/quidups4.txt > ${CustomLists}/temp/quidups5.txt	# Remove all lines with underscore chars
	sed -e 's/^/zone "/g' ${CustomLists}/temp/quidups5.txt > ${CustomLists}/temp/quidups6.txt 	# Replace beginning of line 127.0.0.1  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/quidups6.txt > ${CustomLists}/temp/quidups7.txt # Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };
	
	cat ${CustomLists}/temp/quidups7.txt >> ad-blocker.db

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 1 7 quidups
	fi
fi

if [[ "$fakenews" -eq 1 ]]; then
	# Download blocklist from https://github.com/marktron/fakenews
	wget "https://raw.githubusercontent.com/marktron/fakenews/master/fakenews"
	mv fakenews ${CustomLists}/temp/fakenews1.txt

	sed -e '/^0.0.0.0/!d' ${CustomLists}/temp/fakenews1.txt> ${CustomLists}/temp/fakenews2.txt	# Remove all lines that does not start with 0.0.0.0
	sed 's/#.*//' ${CustomLists}/temp/fakenews2.txt > ${CustomLists}/temp/fakenews3.txt	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/fakenews3.txt > ${CustomLists}/temp/fakenews4.txt	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/_/d' ${CustomLists}/temp/fakenews4.txt > ${CustomLists}/temp/fakenews5.txt	# Remove all lines with underscore chars
	sed -e 's/0.0.0.0 /zone "/g' ${CustomLists}/temp/fakenews5.txt > ${CustomLists}/temp/fakenews6.txt	# Replace beginning of line 0.0.0.0  by zone "
	sed -e 's/$/" { type master; notify no; file "\/etc\/zone\/master\/null.zone.file"; };/g' ${CustomLists}/temp/fakenews6.txt > ${CustomLists}/temp/fakenews7.txt	# Replace end of line \n or LF by " { type master; notify no; file "/etc/zone/master/null.zone.file";  };

	cat ${CustomLists}/temp/fakenews7.txt >> ad-blocker.db	# Add the fakenewslist to the blocklist

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 1 7 fakenews
	fi
fi


# create empty white and blacklists when not present.
if [ ! -f ${CustomLists}/blacklist.txt ]; then
     > ${CustomLists}/blacklist.txt
fi
if [ ! -f ${CustomLists}/whitelist.txt ]; then
     > ${CustomLists}/whitelist.txt
fi


	# New lines \n will be added at the end of the custom Blacklist + Whitelist they can be sorted afterwards.
	# Imported blacklists without \n aka LF at the end will get concatenated and thus not sortable.
	# This prevents the whitelist from working, because only after sorting the double lines can be removed.
	sed 's/#.*//' ${CustomLists}/blacklist.txt > ${CustomLists}/temp/blacklist.clean1.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/blacklist.clean1.txt > ${CustomLists}/temp/blacklist.clean2.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/^\s*$/d' ${CustomLists}/temp/blacklist.clean2.txt > ${CustomLists}/temp/blacklist.clean3.txt 		# Remove all empty lines
	sed -e "s/\r/\n/g" ${CustomLists}/temp/blacklist.clean3.txt > ${CustomLists}/temp/blacklist.clean4.txt
	sed -e "s/\n\n/\n/g" ${CustomLists}/temp/blacklist.clean4.txt > ${CustomLists}/temp/blacklist.clean5.txt
	sed -e s/"  "/" "/g ${CustomLists}/temp/blacklist.clean5.txt > ${CustomLists}/temp/blacklist.clean6.txt
	echo "" >> ${CustomLists}/temp/blacklist.clean6.txt

	sed 's/#.*//' ${CustomLists}/whitelist.txt > ${CustomLists}/temp/whitelist.clean1.txt 	# Remove all after #
	sed 's/ *$//' ${CustomLists}/temp/whitelist.clean1.txt > ${CustomLists}/temp/whitelist.clean2.txt 	# Remove all spaces at the end of the line (Needed after the REMOVE # command)
	sed '/^\s*$/d' ${CustomLists}/temp/whitelist.clean2.txt > ${CustomLists}/temp/whitelist.clean3.txt 		# Remove all empty lines
	sed -e "s/\r/\n/g" ${CustomLists}/temp/whitelist.clean3.txt > ${CustomLists}/temp/whitelist.clean4.txt
	sed -e "s/\n\n/\n/g" ${CustomLists}/temp/whitelist.clean4.txt > ${CustomLists}/temp/whitelist.clean5.txt
	sed -e s/"  "/" "/g ${CustomLists}/temp/whitelist.clean5.txt > ${CustomLists}/temp/whitelist.clean6.txt
	echo "" >> ${CustomLists}/temp/whitelist.clean6.txt

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 1 6 blacklist.clean
    fi 

    if [[ "$Cleanuptemp" -eq 1 ]]; then
        Cleanup 1 6 whitelist.clean
    fi 
    
	# Add the custom Blacklist to the blocklist
	cat ${CustomLists}/temp/blacklist.clean6.txt >> ad-blocker.db
	
	# Replace the capital chars | Sort the Blacklist | Then remove the double ones
	sed -e 's/\([[:upper:]]\)/\L\1/g' ad-blocker.db > ad-blocker.temp1.db # Change capital chars to lowercase
	mv ad-blocker.temp1.db ad-blocker.db
	sort ad-blocker.db | uniq > ad-blocker.temp2.db
	mv ad-blocker.temp2.db ad-blocker.db



	# Add the custom  Whitelist to the blocklist
	cat ${CustomLists}/temp/whitelist.clean6.txt >> ad-blocker.db
	cat ${CustomLists}/temp/whitelist.clean6.txt >> ad-blocker.db
		
	# Sort the blacklist + whitelist | Then remove the double ones (whitelist)
	sort ad-blocker.db | uniq -u > ad-blocker.temp.db
	mv ad-blocker.temp.db ad-blocker.db
    cp ad-blocker.db ${CustomLists}/final-list.txt
    
    
    # Include the new zone data
       if [ -f ad-blocker.db ] && [ -f null.zone.file ]; then

		# Rebuild data null.zone.file
		cd ${ZoneDataDir}
		echo 'zone "null.zone.file" {'					> null.zone.file
		echo '	type master;'							>> null.zone.file
		echo '	file "/etc/zone/master/null.zone.file";'>> null.zone.file
		echo '	allow-transfer {any;};'					>> null.zone.file
		echo '	allow-update {none;};'					>> null.zone.file
		echo '	allow-query {any;};'					>> null.zone.file
		echo '};'										>> null.zone.file
		echo 'include "/etc/zone/data/ad-blocker.db";'	>> null.zone.file
		chown -v DNSServer:DNSServer null.zone.file #>> ${ScriptLog}
		chmod -v 0644 null.zone.file #>> ${ScriptLog}

		# Rebuild master null.zone.file
		cd ${ZoneMasterDir}
		ZoneSN=$(sed -n '/^@/,/^[^;]*)/H;${;x;s/.*@[^(]*([^0-9]*//;s/[^0-9].*//;p;}' 'null.zone.file')
		Now=$(date +%Y%m%d)
		if [[ "${Now}" -ne "${ZoneSN::-2}" ||  ${#ZoneSN} -lt ${#Now} ]]; then
			ZoneSN="${Now}00" #Set new timestamp
		else
			ZoneSN="${Now}$(printf '%02d' $((10#${ZoneSN: -2} + 1)) )" # Increment if already same timestamp
		fi
		echo '$TTL 86400				; one day'				> null.zone.file
		echo '@ IN	SOA	ns.null.zone.file. mail.null.zone.file. (' >> null.zone.file
		echo '			'${ZoneSN}'	; serial number YYYYMMDDNN'	>> null.zone.file
		echo '			86400		; refresh	1 day'			>> null.zone.file
		echo '			7200		; retry		2 hours'		>> null.zone.file
		echo '			864000		; expire	10 days'		>> null.zone.file
		echo '			86400 )		; min ttl	1 day'			>> null.zone.file
		echo '		NS	ns.null.zone.file.'						>> null.zone.file
		echo '		A	127.0.0.1'								>> null.zone.file
		echo '* IN	A	127.0.0.1'								>> null.zone.file
		chown -v DNSServer:DNSServer null.zone.file # >> ${ScriptLog}
		chmod -v 0644 null.zone.file # >> ${ScriptLog}
	fi

    # Reload the server config after modifications
       ${RootDir}/script/reload.sh

    exit 0
